import cattr
from datetime import date, datetime
from dateutil import parser


def date_to_str(value: date) -> str:
    return value.strftime("%Y-%m-%d")


def parse_datetime(text: str, t) -> datetime:
    return parser.parse(
        timestr=text,
        parserinfo=parser.parserinfo(dayfirst=False, yearfirst=True),
    )


def register_cattr_hooks():
    cattr.register_structure_hook(datetime, parse_datetime)



from typing import List

from .auth import BaseAuth
from .import_individu import DataIndividu


class Sdgs(BaseAuth):
    def __init__(
        self,
        username: str,
        password: str,
        api_server="https://sdgsdev.kemendesa.go.id/",
    ):
        super().__init__(api_server=api_server)
        self.token = self.login(username, password)

    def validateNik(
        self,
        nik: str,
    ) -> bool:
        json_data = {"nik": nik}
        res = self.api_post_raw(
            "surveyIndividu/validateNik",
            str,
            json=json_data,
        )
        return bool(res)

    def save_individu(self, individu: DataIndividu, rt: str, rw: str):
        data_individu = individu.make_data(desa=self.token.wilayah, rt=rt, rw=rw)
        json_data = {"jsonResult": data_individu}
        res = self.api_post_raw(
            filename="surveyIndividu/save",
            cl=str,
            json=json_data,
        )
        return res



from enum import Enum
from typing import Any, Dict, Type, TypeVar, Union

E = TypeVar("E", bound=Enum)


def make_str_to_enum(mapping: Dict[str, Any], default_value: Union[str, int] = "1"):
    def str_to_enum(cls: Type[E], val: Union[str, int], t: Type[E] = None) -> E:
        if isinstance(val, int):
            if str(val) in cls:
                return cls(str(val))
            return cls(default_value)
        for key, enum_val in mapping.items():
            if key == val:
                return cls(enum_val)
        return cls(default_value)

    return str_to_enum



import attr
from openpyxl import Workbook
from typing import Any, Dict

from sdgs_tools.utils import parse_range

# from .disabilitas import Disabilitas
from .fasilitas_kesehatan import FasilitasKesehatan
from .penghasilan import Penghasilan

MAPPING = {
    "desa": "desa",
    "I.P103": "nama",
    "I.P104": "jenis_kelamin",
    "I.P105": "tempat_lahir",
    "I.P106": "tanggal_lahir",
    "I.P107": "usia",
    "I.P108": "status_pernikahan",
    "I.P109": "usia_menikah",
    "I.P110": "agama",
    "I.P111": "suku_bangsa",
    "I.P112": "warga_negara",
    "I.P113": "nomor_hp",
    "I.P114": "nomor_whatsapp",
    "I.P115": "alamat_email",
    "I.P116": "alamat_facebook",
    "I.P117": "alamat_twitter",
    "I.P118": "alamat_instagram",
    "I.P119": "aktif_internet",
    "I.P120": "akses_melalui",
    "I.P121": "kecepatan_internet",
    "I.P201": "kondisi_pekerjaan",
    "I.P202": "pekerjaan_utama",
    "I.P202-Comment": "pekerjaan_utama_comment",
    "I.P203": "jamsos_ketenagakerjaan",
    "I.P204": "penghasilan",
    "I.P204_penghasilan": "pekerjaan_penghasilan",
    "I.P401": "penyakit_diderita",
    "I.P402": "fasilitas_kesehatan",
    "I.P403": "jamsos_kesehatan",
    "I.P404": "disabilitas",
    "I.P405": "setahun_melahirkan",
    "I.P406": "mendapat_asi",
    "I.P501": "pendidikan_tertinggi",
    "I.P502": "tahun_pendidikan",
    "I.P503": "pendidikan_diikuti",
    "I.P504": "pelatihan_diikuti",
    "I.P505": "bahasa_permukiman",
    "I.P506": "bahasa_formal",
    "I.P507": "kerja_bakti",
    "I.P508": "siskamling",
    "I.P509": "pesta_rakyat",
    "I.P510": "menolong_kematian",
    "I.P511": "menolong_sakit",
    "I.P512": "menolong_kecelakaan",
    "I.P513": "memperoleh_pelayanan_desa",
    "I.P514": "pelayanan_desa",
    "I.P515": "saran_desa",
    "I.P516": "keterbukaan_desa",
    "I.P517": "terjadi_bencana",
    "I.P518": "terdampak_bencana",
    "kecamatan": "kecamatan",
    "kota": "kota",
    "nik": "nik",
    "no_kk": "no_kk",
    "provinsi": "provinsi",
    "rt": "rt",
    "rw": "rw",
}

NORMAL_DATA = [
    "no_kk",
    "nik",
    "nama",
    "jenis_kelamin",
    "tempat_lahir",
    "tanggal_lahir",
    "usia",
    "status_pernikahan",
    "usia_menikah",
    "agama",
    "suku_bangsa",
    "warga_negara",
    "nomor_hp",
    "nomor_whatsapp",
    "alamat_email",
    "alamat_facebook",
    "alamat_twitter",
    "alamat_instagram",
    "aktif_internet",
    "akses_melalui",
    "kecepatan_internet",
    "kondisi_pekerjaan",
    "pekerjaan_utama",
    "pekerjaan_utama_comment",
    "jamsos_ketenagakerjaan",
    "pekerjaan_penghasilan",
    "penyakit_diderita",
    "jamsos_kesehatan",
    "setahun_melahirkan",
    "mendapat_asi",
    "pendidikan_tertinggi",
    "tahun_pendidikan",
    "pendidikan_diikuti",
    "pelatihan_diikuti",
    "bahasa_permukiman",
    "bahasa_formal",
    "kerja_bakti",
    "siskamling",
    "pesta_rakyat",
    "menolong_kematian",
    "menolong_sakit",
    "menolong_kecelakaan",
    "memperoleh_pelayanan_desa",
    "pelayanan_desa",
    "saran_desa",
    "keterbukaan_desa",
    "terjadi_bencana",
    "terdampak_bencana",
]

PENGHASILAN_DATA = [
    "sumber_penghasilan",
    "penghasilan_comment",
    "penghasilan_jumlah",
    "penghasilan_setahun",
    "penghasilan_diekspor",
]

FASILITAS_KESEHATAN_DATA = [
    "rumah_sakit",
    "rumah_sakit_bersalin",
    "puskesmas_rawat_inap",
    "puskesmas_tanpa_inap",
    "puskesmas_pembantu",
    "poliklinik",
    "tempat_praktik_dokter",
    "rumah_bersalin",
    "tempat_praktik_bidan",
    "poskesdes",
    "polindes",
    "apotik",
    "toko_obat_jamu",
    "posyandu",
    "posbindu",
    "tempat_praktik_dukun",
]


@attr.dataclass
class MappingIndividu:
    no_kk: str = "A"
    nik: str = "B"
    nama: str = "C"
    jenis_kelamin: str = "D"
    tempat_lahir: str = "E"
    tanggal_lahir: str = "F"
    usia: str = "G"
    status_pernikahan: str = "H"
    usia_menikah: str = "I"
    agama: str = "J"
    suku_bangsa: str = "K"
    warga_negara: str = "L"
    nomor_hp: str = "M"
    nomor_whatsapp: str = "N"
    alamat_email: str = "O"
    alamat_facebook: str = "P"
    alamat_twitter: str = "Q"
    alamat_instagram: str = "R"
    aktif_internet: str = "S"
    akses_melalui: str = "T"
    kecepatan_internet: str = "U"
    kondisi_pekerjaan: str = "V"
    pekerjaan_utama: str = "W"
    pekerjaan_utama_comment: str = "X"
    jamsos_ketenagakerjaan: str = "Y"
    penghasilan: str = "Z"  # TODO Penghasilan Dari Sheet Penghasilan
    pekerjaan_penghasilan: str = "AA"
    penyakit_diderita: str = "AB"
    fasilitas_kesehatan: str = "AC-AR"  # TODO Parse From Range
    jamsos_kesehatan: str = "AS"
    disabilitas: str = "AT"  # TODO Parse Multi-Select
    setahun_melahirkan: str = "AU"
    mendapat_asi: str = "AV"
    pendidikan_tertinggi: str = "AW"
    tahun_pendidikan: str = "AX"
    pendidikan_diikuti: str = "AY"
    pelatihan_diikuti: str = "AZ"
    bahasa_permukiman: str = "BA"
    bahasa_formal: str = "BB"
    kerja_bakti: str = "BC"
    siskamling: str = "BD"
    pesta_rakyat: str = "BE"
    menolong_kematian: str = "BF"
    menolong_sakit: str = "BG"
    menolong_kecelakaan: str = "BH"
    memperoleh_pelayanan_desa: str = "BI"
    pelayanan_desa: str = "BJ"
    saran_desa: str = "BK"
    keterbukaan_desa: str = "BL"
    terjadi_bencana: str = "BM"
    terdampak_bencana: str = "BN"
    # Sheet Penghasilan
    sumber_penghasilan: str = "A"
    penghasilan_comment: str = "B"
    penghasilan_jumlah: str = "C"
    penghasilan_setahun: str = "D"
    penghasilan_diekspor: str = "E"
    # Fasilitas Kesehatan
    rumah_sakit: str = "AC"
    rumah_sakit_bersalin: str = "AD"
    puskesmas_rawat_inap: str = "AE"
    puskesmas_tanpa_inap: str = "AF"
    puskesmas_pembantu: str = "AG"
    poliklinik: str = "AH"
    tempat_praktik_dokter: str = "AI"
    rumah_bersalin: str = "AJ"
    tempat_praktik_bidan: str = "AK"
    poskesdes: str = "AL"
    polindes: str = "AM"
    apotik: str = "AN"
    toko_obat_jamu: str = "AO"
    posyandu: str = "AP"
    posbindu: str = "AQ"
    tempat_praktik_dukun: str = "AR"
    _normal_cols: Dict[str, str] = attr.field(factory=dict)
    _penghasilan_cols: Dict[str, str] = attr.field(factory=dict)
    _fasilitas_kesehatan_cols: Dict[str, str] = attr.field(factory=dict)

    def __attrs_post_init__(self):
        # Name : Column
        self._normal_cols: Dict[str, str] = dict()
        for name in NORMAL_DATA:
            col = getattr(self, name)
            assert col
            self._normal_cols[name] = col
        self._penghasilan_cols: Dict[str, str] = dict()
        for name in PENGHASILAN_DATA:
            col = getattr(self, name)
            assert col
            self._penghasilan_cols[name] = col
        self._fasilitas_kesehatan_cols: Dict[str, str] = dict()
        for name in FASILITAS_KESEHATAN_DATA:
            col = getattr(self, name)
            assert col
            self._fasilitas_kesehatan_cols[name] = col

    def __call__(
        self,
        wb: Workbook,
        row: int,
        individu_ws: str = "Individu",
        penghasilan_ws: str = "Penghasilan",
    ) -> Dict[str, Any]:
        data: Dict[str, Any] = dict()
        individu = wb[individu_ws]
        for name, col in self._normal_cols.items():
            data[name] = individu[f"{col}{row}"].value
        # penghasilan: str = "Z"  # TODO Penghasilan Dari Sheet Penghasilan
        penghasilans = individu[f"{self.penghasilan}{row}"].value
        if penghasilans:
            data["penghasilan"] = Penghasilan.make_range(
                ws=wb[penghasilan_ws],
                rows=parse_range(penghasilans),
                cols=self._penghasilan_cols,
            )
        else:
            data["penghasilan"] = Penghasilan.default()
        # fasilitas_kesehatan: str = "AC-AR"  # TODO Parse From Range
        data["fasilitas_kesehatan"] = FasilitasKesehatan.make_row(
            ws=individu,
            row=row,
            cols=self._fasilitas_kesehatan_cols,
        )
        # disabilitas: str = "AT"  # TODO Parse Multi-Select
        data["disabilitas"] = individu[f"{self.disabilitas}{row}"].value
        return data

    def get_nik(
        self,
        wb: Workbook,
        row: int,
        individu_ws: str = "Individu",
    ):
        individu = wb[individu_ws]
        return individu[f"{self.nik}{row}"].value



import attr
import cattr
from typing import Any, Dict, List, Optional
from sdgs_tools.dashboard.enums import (
    Agama,
    AksesInternet,
    JenisKelamin,
    KecepatanInternet,
    KeterbukaanDesa,
    KondisiPekerjaan,
    PekerjaanUtama,
    PelayananDesa,
    Pendidikan,
    StatusPernikahan,
    Warganegara,
    YaTidak,
)

from . import (
    Disabilitas,
    FasilitasKesehatan,
    Penghasilan,
    PenyakitDiderita,
)
from .mapping import MAPPING


@attr.dataclass
class DataIndividu:
    no_kk: str
    nik: str
    nama: str
    jenis_kelamin: JenisKelamin
    tempat_lahir: str
    tanggal_lahir: str
    usia: str
    status_pernikahan: StatusPernikahan
    usia_menikah: str
    agama: Agama
    suku_bangsa: str
    warga_negara: Warganegara
    nomor_hp: str
    nomor_whatsapp: Optional[str]
    alamat_email: Optional[str]
    alamat_facebook: Optional[str]
    alamat_twitter: Optional[str]
    alamat_instagram: Optional[str]
    aktif_internet: YaTidak
    akses_melalui: Optional[AksesInternet]
    kecepatan_internet: KecepatanInternet
    # Pekerjaan
    kondisi_pekerjaan: KondisiPekerjaan
    pekerjaan_utama: Optional[PekerjaanUtama]
    pekerjaan_utama_comment: Optional[str]
    jamsos_ketenagakerjaan: Optional[YaTidak]
    penghasilan: List[Penghasilan]
    pekerjaan_penghasilan: Optional[str]
    # Kesehatan
    penyakit_diderita: PenyakitDiderita
    fasilitas_kesehatan: FasilitasKesehatan
    jamsos_kesehatan: YaTidak
    disabilitas: Disabilitas
    setahun_melahirkan: Optional[YaTidak]
    mendapat_asi: Optional[YaTidak]
    # Pendidikan
    pendidikan_tertinggi: Pendidikan
    tahun_pendidikan: Optional[str]
    pendidikan_diikuti: Optional[Pendidikan]
    pelatihan_diikuti: Optional[str]
    bahasa_permukiman: str
    bahasa_formal: str
    kerja_bakti: str
    siskamling: str
    pesta_rakyat: str
    menolong_kematian: str
    menolong_sakit: str
    menolong_kecelakaan: str
    memperoleh_pelayanan_desa: YaTidak
    pelayanan_desa: Optional[PelayananDesa]
    saran_desa: YaTidak
    keterbukaan_desa: Optional[KeterbukaanDesa]
    terjadi_bencana: YaTidak
    terdampak_bencana: Optional[YaTidak]

    def __attrs_post_init(self):
        if self.terjadi_bencana and self.terdampak_bencana is None:
            raise ValueError(
                "Jika Terjadi bencana mohon diisi terdampak bencana atau tidak"
            )

    def make_data(self, desa: str, rt: str, rw: str):
        raw_data: Dict[str, Any] = cattr.unstructure(self)
        clean_data: Dict[str, Any] = dict()
        for key, properti in MAPPING.items():
            value = raw_data.get(properti)
            if value in (None, "None"):
                # TODO Improve empty value detection
                continue
            clean_data[key] = value
        clean_data.update(
            {
                "desa": desa,
                "kecamatan": desa[0:7],
                "kota": desa[0:4],
                "provinsi": desa[0:2],
                "rt": rt,
                "rw": rw,
            }
        )
        return clean_data



import attr
import cattr
from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, List, Optional

from sdgs_tools.dashboard.enums import Diekspor
from sdgs_tools.dashboard.enums import SumberPenghasilan


@attr.dataclass
class Penghasilan:
    penghasilan_diekspor: Diekspor
    penghasilan_jumlah: str
    penghasilan_setahun: str
    sumber_penghasilan: SumberPenghasilan
    penghasilan_comment: Optional[str] = None

    def todict(self) -> Dict[str, str]:
        data: Dict[str, str] = {
            "diekspor": str(self.penghasilan_diekspor.value),
            "jumlah": str(self.penghasilan_jumlah),
            "penghasilan": str(self.penghasilan_setahun),
            "sumber_penghasilan": str(self.sumber_penghasilan.value),
        }
        if self.sumber_penghasilan == "other":
            if not self.penghasilan_comment:
                raise ValueError("comment harus diisi jika sumber_penghasilan = other")
            data["sumber_penghasilan-Comment"] = self.penghasilan_comment
        return data

    @classmethod
    def default(
        cls,
        penghasilan_diekspor: Diekspor = Diekspor.TIDAK,
        penghasilan_jumlah: str = "0",
        penghasilan_setahun: str = "0",
        sumber_penghasilan: SumberPenghasilan = SumberPenghasilan.LAINNYA,
        penghasilan_comment: str = None,
    ) -> "Penghasilan":
        return cls(
            penghasilan_diekspor=penghasilan_diekspor,
            penghasilan_jumlah=penghasilan_jumlah,
            penghasilan_setahun=penghasilan_setahun,
            sumber_penghasilan=sumber_penghasilan,
            penghasilan_comment=penghasilan_comment,
        )

    @classmethod
    def from_range(cls, ws: Worksheet, rows: List[int], cols: Dict[str, str]):
        results: List[Any] = list()
        for row in rows:
            results.append(
                cls.from_row(
                    ws=ws,
                    row=row,
                    cols=cols,
                )
            )
        return results

    @classmethod
    def from_row(cls, ws: Worksheet, row: int, cols: Dict[str, str]) -> "Penghasilan":
        data: Dict[str, Any] = dict()
        for name, col in cols.items():
            data[name] = ws[f"{col}{row}"].value
        return cattr.structure(data, cls)

    @staticmethod
    def make_range(ws: Worksheet, rows: List[int], cols: Dict[str, str]):
        results: List[Any] = list()
        for row in rows:
            data: Dict[str, Any] = dict()
            for name, col in cols.items():
                data[name] = ws[f"{col}{row}"].value
            results.append(data)
        return results


cattr.register_unstructure_hook(Penghasilan, Penghasilan.todict)



import tkinter as tk
from tkinter.messagebox import showinfo

from sdgs_tools import __version__
from . import ContentFrame
from . import SidebarFrame

from . import AplikasiTab
from . import DashboardTab


class MainApp(tk.Tk):
    def __init__(self):
        super().__init__(screenName="SDGS Tools")
        self.title(f"SDGS Tools v{__version__}")
        self.sidebar = SidebarFrame(self)
        self.content = ContentFrame(self)

        self.aplikasi_tab = AplikasiTab(self.content)
        self.dashboard_tab = DashboardTab(self.content)
        self.sidebar.add_button("Aplikasi SDGS", self.aplikasi_tab.tkraise)
        self.sidebar.add_button("Dashboard SDGS", self.dashboard_tab.tkraise)

        self.sidebar.add_button("Tentang", self.about)

    def __call__(self):
        self.mainloop()

    def about(self):
        showinfo(
            "Tentang",
            f"SDGS-Tools versi{__version__} dibuat oleh Habib Rohman\n"
            "Donasi : https://saweria.co/hexatester",
        )



from enum import Enum
from . import make_str_to_enum

MAPPING = {
    "Baik": "1",
    "Cukup": "2",
    "Buruk": "3",
}


class PelayananDesa(Enum):
    BAIK = "1"
    CUKUP = "2"
    BURUK = "3"

    from_str = classmethod(make_str_to_enum(MAPPING, "1"))



import attr
import cattr
from typing import Dict

from sdgs_tools.dashboard.enums import YaTidak


MAPPING = {
    "Tunanetra": "tunanetra",
    "Tunarungu": "tunarungu",
    "Tunawicara": "tunawicara",
    "Tunarungu Wicara": "tunarungu_wicara",
    "Tunadaksa": "tunadaksa",
    "Tunagrahita": "tunagrahita",
    "Tunalaras": "tunalaras",
    "Cacat Eks Kusta": "cacat_eks_kusta",
    "Cacat Ganda": "cacat_ganda",
    "Dipasung": "dipasung",
}


@attr.dataclass
class Disabilitas:
    tunanetra: YaTidak = YaTidak.TIDAK
    tunarungu: YaTidak = YaTidak.TIDAK
    tunawicara: YaTidak = YaTidak.TIDAK
    tunarungu_wicara: YaTidak = YaTidak.TIDAK
    tunadaksa: YaTidak = YaTidak.TIDAK
    tunagrahita: YaTidak = YaTidak.TIDAK
    tunalaras: YaTidak = YaTidak.TIDAK
    cacat_eks_kusta: YaTidak = YaTidak.TIDAK
    cacat_ganda: YaTidak = YaTidak.TIDAK
    dipasung: YaTidak = YaTidak.TIDAK

    def todict(self) -> Dict[str, str]:
        return {
            "1": "1" if self.tunanetra else "2",
            "2": "1" if self.tunarungu else "2",
            "3": "1" if self.tunawicara else "2",
            "4": "1" if self.tunarungu_wicara else "2",
            "5": "1" if self.tunadaksa else "2",
            "6": "1" if self.tunagrahita else "2",
            "7": "1" if self.tunalaras else "2",
            "8": "1" if self.cacat_eks_kusta else "2",
            "9": "1" if self.cacat_ganda else "2",
            "10": "1" if self.dipasung else "2",
            "11": "2",
            "12": "2",
            "13": "2",
            "14": "2",
            "15": "2",
            "16": "2",
            "17": "2",
            "18": "2",
            "19": "2",
        }

    @classmethod
    def from_str(cls, val: str, t=None):
        if not val:
            return cls()
        data: Dict[str, YaTidak] = dict()
        for key, name in MAPPING.items():
            if key in val:
                data[name] = YaTidak.YA
            else:
                data[name] = YaTidak.TIDAK
        return cls(**data)


cattr.register_unstructure_hook(Disabilitas, Disabilitas.todict)
cattr.register_structure_hook(Disabilitas, Disabilitas.from_str)
